#include "Maiden_struct.h"
#include "Maiden_funct.h"
#include "Constants.h"
#include <math.h>
#include <stdio.h>
#ifdef __APPLE__
#include "malloc.h"
#else
#include <malloc.h>
#endif

int soil_evap(const param_struct *par, soilev_struct *soilev, meteo_struct *meteo, water_struct *water, soil_struct *soil, radiation_struct *rad, int i)
{
	int ok=1;
	double t,t1,t2,tk,Rv,surftk,r_soill;
	double se,ss; //soil evaporation and snow evaporation
	double Epot,Epot_snow;
	double swp,soil_rhl,air_d,sat_mrl,sm,sat,snow,f_snow;
	double ra; //aerodynamic resistance to water vapour transfer, difficult to estimate
	double gamma,s,pvs1,pvs2,lhvap;
	double albedo; //snow albedo PAD 07092019
	// double dt = 0.2;     /* set the temperature offset for slope calculation */ FG

	/* assigning local variables */
	t = meteo->s_tday[i];
	tk = t + 273.15;
	surftk = meteo->s_surft[i] + 273.15;// daily air t averaged over 30 previous days

	/* gas constant */
	Rv = R / MW; //from (m3 Pa mol-1 K-1) to [J.Kg-1 h2o K-1]

	/* air density */
	air_d = meteo->s_pa[i]/(tk*Rv); // Pa/(t Kelvin * J Kg-1 h2o K-1)
	//air_d = air_d * 1000.; // from kg m-3 to g m-3
	//printf("%d meteo->s_pa %lf and t %lf\n",i,meteo->s_pa[i],tk-273.15);
	//printf("Rv= %lf and air density %lf\n",Rv,air_d);
	//printf(" %lf et masse vol air %lf (kg/m3)\n",meteo->s_tday[i],AirVolumicMass(tk,meteo->rh[i],meteo->s_pa[i]));

	/* saturated mixing ratio */
	water->sat_mr[i] = sat_mrl = meteo->s_svp[i]/meteo->s_pa[i]; // Pa/Pa
	
	/* soil relative humidity */
	if (i==0)
	  {
		if (meteo->s_tavg[i] < T_F)
		  {
			swp = -100000.;
			sm = 0.01; //minimum
			snow = 50; //mm
			f_snow = 1.;
		  }
		else
		  {
			swp = -50.;
			sm = 0.489 - 0.00126*par->sand1;//saturation
			snow = 0; //mm
			f_snow = 0.;
		  }
		swp = -50.;
		sm = 0.489 - 0.00126*par->sand1;//saturation
		snow = 50; //mm
		f_snow = 1.;
	  }
	else
	  {
		swp = soil->swp[0]/1000.; // from mm to m water
		sm = water->sm[0][i];
		snow = water->icesnow[i]+water->h2osnow[i];
		f_snow = My_min(1.,water->dzsnow[i]/0.1);
	  }
	
	/*
	    Soil Relative Humidity :
	    Mathieu et al., 1996 (Eq.8)
	    Baud et al., 2005 (Eq.46)   (seems as below)
	    Stewart et al., 1999 (Eq.1.1)
	    G_STD=9.81 [m/s2] std. grav. accel. Constants.h
	*/

	water->soil_rh[i] = soil_rhl = exp(G_STD * swp/(Rv * surftk)); //swp in m of water, surftk in K

	//printf("%d %lf (soil_evap.cpp)\n",i,water->soil_rh[i]);
	if (water->soil_rh[i]>1.) water->soil_rh[i] = 1.;
	if (water->soil_rh[i]<0.) water->soil_rh[i] = 0.;

	/* soil resistance misson 112305 */
	sat = 0.489 - 0.00126*par->sand1;//saturated volumetric soil water content (porosity)
	r_soill = 3.8113e6*exp(-13.515*sm/sat);//Baldocchi //It goes back to Passerat 1986 (thesis), which exists in paper only

	if (r_soill <= 100.0) r_soill = 100.0;

	if (water->soil_mm[i-1] < par->soilwmin)  r_soill = 10E6;//misson 12105

	if (r_soill>10E6) r_soill = 10E6;//misson 12105
	water->r_soil[i] = r_soill;

	//PAD 180618 (Ignacio)
	//Evaporation is not the same depending on whether it comes from snow or soil, so we need to track both
	t1 = t+0.2;
    t2 = t-0.2;
	pvs1 = 610.7 * exp(17.38 * t1 / (239.0 + t1));
	pvs2 = 610.7 * exp(17.38 * t2 / (239.0 + t2));
	s = (pvs1-pvs2) / (t1-t2);
	lhvap = 2.5023e6 - 2430.54 * t;
	gamma = CP * meteo->s_pa[i] / (lhvap * EPS);
	//ra (s/m) I had most trouble with this, everybody uses a different formula
	ra = par->rad_trans; //resistance to evaporative vapour transfer
	// ra = 400.; // Stigter et al. 2018, I THINK IT IS TOO LARGE
	// ra = 4.72/(1+0.54*u)*pow(log(8*z/h),2); //Thom and Oliver 1977, h is canopy height and z the height where wind speed u is measured
	// ra = 79.4 * pow(u,-0.8); //Nakai, Sakamoto and Terajima 1994
	// ra = 1/B/u_f + u/u_f Blanken and Black 2004 u is wind speed and u_x is friction velocity
		// z_0 = 0.1 * h; // h is canopy height
		// d = displacement height, or height of zero wind speed (2/3 to 3/4 of canopy height)
		// u_x = 0.41 * u[i] / log ((2-d)/z_0);
	
	//snow albedo dependent on snow density, see Greuell and Konzelmann (1994)
	if( i==0 ) //snow not initialized yet
		albedo = 0.58 + (316.861 - 917.) * ( 0.85 - 0.58 ) / ( 316.861 - 917. );
	else
		albedo = 0.58 + (water->icesnow[i]/water->dzsnow[i] - 917.) * ( 0.85 - 0.58 ) / ( 316.861 - 917. );
	  	
	/* lhvap*Epot ([kg/m3]*[J/kg/K]*[Pa]/[s/m]) / [Pa/K] <=> [W/m2]  */
	Epot_snow = (( (1.-albedo) * s * meteo->s_srad[i] ) + ( air_d * CP * meteo->s_hum[i] / ra ) ) / ( s + gamma );  //Penman-Monteith equation
	//Epot_snow = (( air_d * CP * meteo->s_hum[i] / ra ) ) / ( s + gamma );  //Penman-Monteith equation
    Epot_snow = Epot_snow / lhvap; // (kg m-2 s-1) 
		//suppose we evaporate liquid water present in the snow, in soih2o we evaporate this water
		// and transform the remaining Epot with lhvap/lhsub to sublimate snow
	ss = Epot_snow * meteo->s_dayl[i]; // dm3/m2/d <=> 10-3m3/m2/d <=> 10-3m/d <=> mm/d (divided by an implicit water density = 1 kg mm-1 m-2)
	//ss = 0.; //ATTENTION!!!! TEST FOR NO SNOW SUBLIMATION
	
	/* Epot [kg/m3]*[Pa/PA]*[1]/[s/m]  */
	Epot = air_d*sat_mrl*(soil_rhl-meteo->rh[i])/(r_soill); //kg m-2 s-1
	se = Epot * meteo->s_dayl[i]; // dm3/m2/d <=> 10-3m3/m2/d <=> 10-3m/d <=> mm/d (divided by an implicit water density = 1 kg mm-1 m-2)
	
	//printf("%3d %lf %lf (soil_evp73)\n",meteo->yday[i],soil_rhl,meteo->rh[i]);
	//printf("%3d %lf %lf %lf mm (soil_evp73)\n\n",meteo->yday[i],Epot,se,water->sm[0][i]*1000.*par->thick1);
	
	if ( (1.-f_snow)*se + f_snow*ss > sm*1000.*par->thick1 + snow) // PAD 160619 Changed water->sm[0][i] by sm, 0.1 by par->thick1
	  {
		printf("soil_evap 114: soil evap + snow subl > water->sm[0][i] + snow \n");
		printf("day %d se + ss = %lf > sm + snow = %lf + %lf [mm]\n\n",i,(1.-f_snow)*se+f_snow*ss,sm*1000.*par->thick1,snow); 
	  }

	if (se < 0.0) se = 0.0; //misson 211105
	if (se > 0.5*sm*1000*par->thick1) se = 0.5*sm*1000*par->thick1; //misson 211105
	if (ss < 0.0) ss = 0.0;
	//if (ss > snow ) ss = 0.5*snow; //I think this is not necessary

	water->Epot_se[i] = Epot;//mm d-1 //(Ignacio) how is this mm/d? this is in kg.m-2.s-1. Fortunately this is not used anywhere, it is just model output
	water->se[i] = se; // mm/d
	water->ss[i] = ss; // mm/d
	soilev->aird[i] = air_d;
	soilev->satmrl[i] = sat_mrl;
	soilev->soilrhl[i] = soil_rhl;
	soilev->rsoil[i] = r_soill;
	soilev->ra[i] = 100;

	//printf("%lf %lf\n",soilev->ra[i],soilev->rsoil[i]);
	//if (i > 1900) {printf("%i %lf %lf %lf %lf %lf %lf %lf\n",meteo->yday[i],air_d,sat_mrl,soil_rhl,meteo->rh[i],r_soill,r_a,Epot);scanf("%c");}

	return (!ok);
}

